package se.chalmers.cs.gf.tramdemo;

import se.chalmers.cs.gf.tramdemo.query.*;

import se.chalmers.cs.gf.util.Pair;
import static se.chalmers.cs.gf.util.Pair.pair;

import java.awt.Point;
import java.util.*;

public class FindPlace {

        private CoordMap<String> coordMap;

        private String here;

        public FindPlace(CoordMap<String> coordMap, String here) {
                this.coordMap = coordMap;
                this.here = here;
        }

        public List<String> findPlace(Place p) {
                return p.accept(new PlaceResolver(), null);
        }

        private class PlaceResolver implements Place.Visitor<List<String>, Object> {
                public List<String> visit(PStop p, Object arg) {
                        return Arrays.<String>asList(getStopName(p.stop_));
                }
                public List<String> visit(PNamed p, Object arg) {
                        // FIXME: check if this stop exists
                        return Arrays.<String>asList(p.string_);
                }
                public List<String> visit(PHere p, Object arg) {
                        return Arrays.<String>asList(here);
                }
                public List<String> visit(PClick p, Object arg) {
                        return resolveClick(p.click_);
                }
        }

        private String getStopName(Stop stop) {
                return stop.getClass().getSimpleName();
        }

        private List<String> resolveClick(Click c) {
                return c.accept(new ClickResolver(), null);
        }

        private class ClickResolver implements Click.Visitor<List<String>, Object> {
                public List<String> visit(CCoord p, Object arg) {
                        int x = p.integer_0;
                        int y = p.integer_1;
                        List<String> rs = coordMap.get(new Point(x,y));

                        // FIXME: special treatment if there is no stop for the click?
                        if (rs.isEmpty())
                                System.err.println("No stop at (" + x + "," + y + ")");

                        return rs;
                }
                public List<String> visit(CStops p, Object arg) {
                        List<String> names = getStops(p.liststring_);
                        // FIXME: special treatment if there is no stop for the click?
                        if (names.isEmpty())
                                System.err.println("Click did not correspond to a stop");
                        return names;
                }
        }

        private List<String> getStops(ListString ss) {
                return ss.accept(new ListStringToList(), null);
        }

        private class ListStringToList implements ListString.Visitor<List<String>, Object> {
                public List<String> visit(ConsString p, Object arg) {
                        List<String> rest = p.liststring_.accept(this, arg);
                        rest.add(0, p.string_);
                        return rest;
                }
                public List<String> visit(BaseString p, Object arg) {
                        return new LinkedList<String>();
                }
        }

}
